/************************************************************
 * OnlineCoinCoin, by Chrisix (chrisix@gmail.com)
 * Fonctions de gestion des options de configuration utilisateur
 ************************************************************/

var TYPE_INT = "int";
var TYPE_STR = "string";
var TYPE_BOOL = "bool";
var TYPE_CHOICE = "choice";
var TYPE_MULTI_CHOICE = "multichoice";

function Option(descr, value, type) {
    this.descr = descr;
    this.value = value;
    this.type = type;
}

var settings = {
    options: {
        'active_boards': {
            descr: "Tribunes actives",
            type: TYPE_MULTI_CHOICE
        },
        'pinni_size': {
            descr: "Nombre max de posts dans le pinnipède",
            type: TYPE_INT
        },
        'default_ua': {
            descr: "User-agent",
            type: TYPE_STR
        },
        'totoz_server': {
            descr: "Serveur de totoz",
            type: TYPE_STR
        },
        'totoz_mode': {
            descr: "Affichage des totoz",
            type: TYPE_CHOICE
        },
        'default_login': {
            descr: "Login par défaut",
            type: TYPE_STR
        },
        'window_title': {
            descr: "Titre de la fenêtre",
            type: TYPE_STR
        },
        'boss_mode': {
            descr: "Boss-mode",
            type: TYPE_CHOICE
        },
        'style': {
            descr: "Style",
            type: TYPE_CHOICE
        },
	'plonk': {
            descr: "Plonk-list",
            type: TYPE_STR
	}
    },
    setDefault: function () {
        for (var opt in this.options) {
            this.options[opt].value = eval('DEFAULT_'+opt.toUpperCase());
            // _log.debug("Option "+opt+" set to "+this.value(opt));
        }
    },
    set: function(name, val) {
        this.options[name].value = val;
    },
    value: function (name) {
        // // _log.error("name="+name);
        return this.options[name].value;
    },
    save: function () {
        var tab = new Array();
        for (var opt in this.options) {
            var val = this.value(opt);
            if (this.options[opt].type == TYPE_MULTI_CHOICE) {
                // alert('opt: '+opt+' val:'+val);
                val = val.join("|");
            }
            tab.push(opt+"="+val);
            // _log.info("Saving opt: "+opt+" = "+val);
        }
        setCookie("settings", tab.join("\n"), 10000);
    },
    load: function () {
        var settings = getCookie("settings");
        if (settings) {
            var pairs = settings.split("\n");
            for (var i=pairs.length; i--;) {
                var opt_val = pairs[i];
                // _log.debug("found pair: "+opt_val);
                var eqpos = opt_val.indexOf('=');
                var name = opt_val.substr(0, eqpos);
                var opt = this.options[name];
                var val = opt_val.substr(eqpos+1, opt_val.length);
                // _log.debug("name="+name+", val="+val);
                switch (opt.type) {
                  case TYPE_INT:
                  case TYPE_BOOL:
                    val = eval(val);
                    break;
                  case TYPE_MULTI_CHOICE:
                    val = val.split("|");
                    break;
                }
                opt.value = val;
                // _log.debug("Option '"+name+"' loaded with value: "+this.value(name));
            }
        }
    }
}

// Bascule affichage/fermeture du panneau de config
function toggleConfig() {
    var configPanel = document.getElementById("config");
    // _log.debug("current panel= "+configPanel);
    if (configPanel.style.display != 'none') {
        configPanel.style.display = 'none';
    }
    else {
        createConfigPanel(configPanel);
        configPanel.style.display = 'block';
    }
}

// affichage du panneau de config
function dispConfig() {
    var configPanel = document.getElementById("config");
    createConfigPanel(configPanel);
    configPanel.style.display = 'block';
}

// fermeture du panneau de config
function closeConfig() {
    var configPanel = document.getElementById("config");
    configPanel.style.display = 'none';
    // removage des vues
    for (var name in GlobalBoards) {
        var board = GlobalBoards[name];
        if (board.state != STATE_LOADED) {
            board.removeView(document.getElementById('config-'+board.name));
        }
    }
}

function saveConfig() {
    for (var opt in settings.options) {
        var cur_opt = settings.options[opt];
        var opt_elem = document.getElementById('config-'+opt);
        switch (cur_opt.type) {
          case TYPE_INT:
          case TYPE_BOOL:
            cur_opt.value = eval(opt_elem.value);
            break;
          case TYPE_STR:
          case TYPE_CHOICE:
            cur_opt.value = opt_elem.value;
            if (opt == 'window_title') {
                document.title = cur_opt.value;
            }
            break;
          case TYPE_MULTI_CHOICE:
            if (opt == 'active_boards') {
                var res = new Array();
                for (var name in GlobalBoards) {
                    if (GlobalBoards[name].state != STATE_LOADED) {
                        res.push(name);
                    }
                }
                cur_opt.value = res;
            }
            break;
        }
    }
    settings.save();
    closeConfig();
}


// Ajoute une ligne dans le tableau de configuration des tribunes actives
function addConfigLine(board, subpanel) {
    // var subpanelwrapper = document.getElementById('boards-table-wrapper');
    tr = document.createElement('tr');
    tr.setAttribute('id', 'config-'+board.name);
    tr.className = 'subpanel'; // setAttribute('class', "subpanel");
    // Cellule nom de la tribune
    td = document.createElement('td');
    td.className = 'panel-board tab-'+board.name; // setAttribute('class', 'panel-board');
    td.innerHTML = board.name;
    // td.style.background = board.color;
    tr.appendChild(td);
    // Cellule bouton start
    td = document.createElement('td');
    icon = (board.state == STATE_STOP) ? "start.png" : "greystart.png";
    td.innerHTML = '<img id="but-start-'+board.name+'" src="img/'+icon+'" alt="[Démarrer]" title="Démarrer" onclick="BoardStart(GlobalBoards['+"'"+board.name+"'"+'])" />';
    tr.appendChild(td);
    // Cellule bouton stop
    td = document.createElement('td');
    icon = (board.state == STATE_STOP) ? "greystop.png" : "stop.png";
    td.innerHTML = '<img id="but-stop-'+board.name+'" src="img/'+icon+'" alt="[Arrêter]" title="Arrêter" onclick="BoardStop(GlobalBoards['+"'"+board.name+"'"+'])" />';
    tr.appendChild(td);
    // Cellule bouton config
    td = document.createElement('td');
    td.innerHTML = '<img id="but-config-'+board.name+'" src="img/bconfig.png" alt="[Paramètres]" title="Paramètres" onclick="configBoard('+"'"+board.name+"'"+')" />';
    tr.appendChild(td);
    // Cellule bouton remove
    td = document.createElement('td');
    td.innerHTML = '<img id="but-remove-'+board.name+'" src="img/remove.png" alt="[Supprimer]" title="Supprimer" onclick="configRemove('+"'"+board.name+"'"+')" />';
    tr.appendChild(td);
    // Cellule nombre de posts
    td = document.createElement('td');
    td.setAttribute('id', "nbposts-"+board.name);
    td.className = 'cinfo';
    td.innerHTML = board.nbPosts + " posts";
    tr.appendChild(td);
    // Cellule état
    td = document.createElement('td');
    td.setAttribute('id', "cstate-"+board.name);
    td.className = 'cstate';
    td.innerHTML = "["+board.state+"]";
    tr.appendChild(td);
    // Ajout de la ligne
    subpanel.appendChild(tr);
    tr.notified = function (notif, state) {
        var name = this.getAttribute('id').substr(7);
        var board = GlobalBoards[name];
        switch (notif) {
          case NOTIF_STATE:
            document.getElementById("cstate-"+name).innerHTML = "["+board.state+"]";
            switch (state) {
              case STATE_LOADED:
                // self destruction
                break;
              case STATE_STOP:
                document.getElementById("but-start-"+name).src = "img/start.png";
                document.getElementById("but-stop-"+name).src = "img/greystop.png";
                break;
              default:
                document.getElementById("but-start-"+name).src = "img/greystart.png";
                document.getElementById("but-stop-"+name).src = "img/stop.png";
                break;
            }
            break;
          case NOTIF_NEW_POST:
            document.getElementById("nbposts-"+name).innerHTML = board.nbPosts + " posts";
            break;
        }
    };
    board.addView(tr);
    // subpanelwrapper.innerHTML = '<table id="boardsZone" class="subpanel">'+subpanel.innerHTML+'</table>';
}

// Creation du panneau de config
function createConfigPanel(panel) {
    // // _log.error("oldpanel= "+oldpanel);
    // var panel = document.createElement('div');
    // var oldpanel = document.getElementById("panel");
    // document.getElementsByTagName("body")[0].replaceChild(panel, oldpanel);
    // panel.setAttribute('id', "config");
    // panel.setAttribute('class', "config-panel");
    panel.innerHTML = '';
    
    // Section config générale
    var head1 = document.createElement('div');
    head1.className = 'panel-header'; // setAttribute('class', "panel-header");
    head1.innerHTML = 'OnlineCoinCoin '+VERSION+' - Configuration';
    head1.innerHTML += ' <img src="img/closeok.png" alt="[Ok]" title="Enregistrer les changements et fermer" onclick="saveConfig()" />';
    head1.innerHTML += ' <img src="img/cancel.png" alt="[Annuler]" title="Annuler les changements et fermer" onclick="closeConfig()" />';
    panel.appendChild(head1);
    // settings.configPanel(panel);
    // var subpanelwrapper = document.createElement('span');
    var subpanelwrapper = document.createElement('table');
    var subpanel = document.createElement('tbody');
    subpanelwrapper.appendChild(subpanel);
    subpanel.setAttribute('id', "configZone");
    subpanel.setAttribute('class', "subpanel");
    var opt = null;
    for (opt in settings.options) {
        var cur_opt = settings.options[opt];
        var size = 50;
        switch (cur_opt.type) {
          case TYPE_INT:
            size = 3;
          case TYPE_STR:
            subpanel.appendChild(TextBox('config-'+opt, cur_opt.descr, cur_opt.value, size));
            break;
          case TYPE_BOOL:
            subpanel.appendChild(CheckBox('config-'+opt, cur_opt.descr, cur_opt.value));
            break;
          case TYPE_CHOICE:
            var tab = null;
            switch (opt) {
              case 'totoz_mode':
                tab = [TOTOZ_POPUP, TOTOZ_INLINE];
                break;
              case 'boss_mode':
                tab = [BOSSMODE_RANDOM, BOSSMODE_PTRAMO, BOSSMODE_KERVIEL, BOSSMODE_PBPG, BOSSMODE_DECIDEUR]
                break;
              case 'style':
                tab = ['default', 'lefttabs', 'sfw', 'golcc'];
                break;
            }
            if (tab) {
                subpanel.appendChild(SelectBox('config-'+opt, cur_opt.descr, tab, cur_opt.value, size));
            }
            break;
        }
    }
    // panel.appendChild(subpanel);
    // subpanelwrapper.innerHTML = '<table id="configZone" class="subpanel">'+subpanel.innerHTML+'</table>';
    panel.appendChild(subpanelwrapper);
    
    // Section tribunes
    var head2 = document.createElement('div');
    head2.className = 'panel-header'; // setAttribute('class', "panel-header");
    head2.innerHTML = 'Tribunes activées ';
    panel.appendChild(head2);
    // Liste des tribunes disponibles
    var triblist = document.createElement('select');
    triblist.setAttribute('id', "availableboards");
    // var subpanelwrapper = document.createElement('span');
    // subpanelwrapper.setAttribute('id', 'boards-table-wrapper');
    var subpanelwrapper = document.createElement('table');
    var subpanel = document.createElement('tbody');
    subpanelwrapper.appendChild(subpanel);
    subpanel.setAttribute('id', "boardsZone");
    subpanel.setAttribute('class', "subpanel");
    for (name in GlobalBoards) {
        var board = GlobalBoards[name];
        if (board.state != STATE_LOADED) {
            addConfigLine(board, subpanel);
        }
        else {
            addAvailableBoard(triblist, name);
        }
    }
    panel.appendChild(subpanelwrapper);
    // panel.appendChild(subpanel);
    // subpanelwrapper.innerHTML = '<table id="boardsZone" class="subpanel">'+subpanel.innerHTML+'</table>';
    
    // Section tribunes disponibles
    var head3 = document.createElement('div');
    head3.className = 'panel-header'; // setAttribute('class', "panel-header");
    head3.innerHTML = 'Autres tribunes disponibles ';
    panel.appendChild(head3);
    subpanel = document.createElement('div');
    subpanel.appendChild(triblist);
    subpanel.innerHTML += '<img src="img/addboard.png" alt="[+]" title="Ajouter cette tribune" onclick="addNewBoard()" />';
    panel.appendChild(subpanel);
}

// Ajoute une entrée dans la liste des tribunes disponibles non actives
function addAvailableBoard(list, name) {
    var opt = document.createElement('option');
    opt.setAttribute('value', name);
    opt.innerHTML = name;
    list.appendChild(opt);
}

// Affiche le panneau de configuration pour la tribune "name"
function configBoard(name) {
    var board = GlobalBoards[name];
    var panel = board.configPanel();
    board.tmpcookieback = board.cookie;
    document.getElementsByTagName("body")[0].appendChild(panel);
}

// Enlève une ligne dans le tableau de configuration des tribunes actives
function configRemove(name) {
    var board = GlobalBoards[name];
    var tab = GlobalBoardTabs[name];
    var line = document.getElementById('config-'+name);
    board.removeView(line);
    board.stop();
    document.getElementById('boardsZone').removeChild(line);
    tab.removeTab()
    addAvailableBoard(document.getElementById('availableboards'), name);
}

// Sauvegarde la configuration de la tribune "name" et ferme son panneau de config
function saveBoardConfig(name) {
    var board = GlobalBoards[name];
    board.color = document.getElementById('config-color').value;
    board.updateCSS(); // change couleur style de la board
    board.alias = document.getElementById('config-alias').value;
    board.login = document.getElementById('config-login').value;
    board.cookie = document.getElementById('config-cookie').value;
    if (board.tmpcookieback != board.cookie) { board.tmpcookie = false; }
    // board.plonk = document.getElementById('config-plonk').value;
    board.delay = parseInt(document.getElementById('config-delay').value)*1000;
    board.saveConfig();
    cancelBoardConfig(name);
}

// Ferme le panneau de config de la tribune "name"
function cancelBoardConfig(name) {
    var panel = document.getElementById("config-panel-"+name);
    if (panel) {
        document.getElementsByTagName("body")[0].removeChild(panel);
    }
}
