/************************************************************
 * OnlineCoinCoin, by Chrisix (chrisix@gmail.com)
 * Définition de constantes, variables globales et fonctions utilitaires
 ************************************************************/
 
// Constantes globales génériques
var VERSION = '0.2.1';

// Constantes de gestion des logs
var LOG_DEBUG = 5;
var LOG_INFO = 4;
var LOG_WARNING = 3;
var LOG_ERROR = 2;
var LOG_CRITICAL = 1;
var LOG_NONE = 0;
var LOG_NAMES = ['None', 'Critical', 'Error', 'Warning', 'Info', 'Debug'];
var LOG_MAX_SIZE = 1024; // parce que bon, faut pas pousser non plus

// Constantes de gestion des états
var STATE_LOADED = 'loaded'; // Tribune chargée
var STATE_STOP = 'stop';     // Tribune arrêtée
var STATE_IDLE = 'idle';     // Tribune en attente
var STATE_HTTP = 'http';     // Tribune en train de rapatrier une url
var STATE_PARSE = 'parsing';   // En train de parser le backend
var STATE_POST = 'posting';  // En train de poster
var STATE_HTTP_ERROR = 'http-error';   // Erreur réseau
var STATE_PARSE_ERROR = 'parse-error'; // Erreur au parsing du backend
var STATE_POST_ERROR = 'post-error';   // Erreur au post

// Modes d'affichage des totoz
var TOTOZ_POPUP = 'popup';
var TOTOZ_INLINE = 'inline';

// Types de slip
var SLIP_TAGS_RAW = 'Raw tags';
var SLIP_TAGS_ENCODED = 'Tags encoded';

// Boss-mode
var BOSSMODE_RANDOM = 'Aléatoire';
var BOSSMODE_PTRAMO = 'Pierre Tramo';
var BOSSMODE_KERVIEL = 'J. Kerviel';
var BOSSMODE_PBPG = 'pBpG';
var BOSSMODE_DECIDEUR = 'Diçaïdor'

// Valeurs de config par défaut
var DEFAULT_LOG_LEVEL = LOG_DEBUG;
var DEFAULT_DEFAULT_UA = "onlineCoinCoin/"+VERSION;
var DEFAULT_PINNI_SIZE = 500;
var DEFAULT_AUTOREFRESH = true;
var DEFAULT_TIMEOUT = 30000;
var DEFAULT_ACTIVE_BOARDS = new Array();
var DEFAULT_TOTOZ_SERVER = "http://sfw.totoz.eu/";
var DEFAULT_TOTOZ_MODE = TOTOZ_POPUP;
var DEFAULT_DEFAULT_LOGIN = '';
var DEFAULT_WINDOW_TITLE = "OnlineCoinCoin";
var DEFAULT_BOSS_MODE = BOSSMODE_RANDOM;
var DEFAULT_STYLE = 'default';
var DEFAULT_PLONK = '';

// Notifications
var NOTIF_STATE = 'change state';
var NOTIF_NEW_POST = 'new post';
var NOTIF_ANSWER = 'new answer';
var NOTIF_BIGORNO = 'bigorno';

var is_ie = true;
try {
  var plop = new ActiveXObject('Microsoft.XMLHTTP');
}
catch (err) {
  is_ie = false;
}

// Renvoie une classe CSS (permet de modifier ensuite le style de la classe
// dans sa globalité, affectant ainsi tous les éléments de cette classe)
function getStyleClass (className) {
    if (document.styleSheets.length < 1) {
        return null;
    }
    if (document.styleSheets[0].cssRules) {
        var cssRules = 'cssRules';
    } else {
        var cssRules = 'rules';
    }
    for (var s=document.styleSheets.length; s--;) {
        if (!document.styleSheets[s].disabled) {
            for (var r=document.styleSheets[s][cssRules].length; r--;) {
                if (document.styleSheets[s][cssRules][r].selectorText == '.' + className) {
                    return document.styleSheets[s][cssRules][r];
                }
            }
        }
    }
    return null;
}

// Ajoute dynamiquement de nouvelles définitions de classes CSS globales
function addCSSClass(ruleName, cssText) {
    if (document.styleSheets) {
        if (!getStyleClass(ruleName)) {
            for (var s=document.styleSheets.length; s--;) {
                // if (!document.styleSheets[s].disabled) {
                    if (document.styleSheets[s].addRule) {
                        document.styleSheets[s].addRule('.'+ruleName, cssText, 0); // IE style
                    } else {
                        document.styleSheets[s].insertRule('.'+ruleName+' {'+cssText+'}', 0); // Moz style.
                    }
                // }
            }
        }
    }
    return getStyleClass(ruleName);
}

// Ajoute de nouvelles classes CSS à un élément particulier
function addClass(node, newClass) {
    var theClass = node.className; // getAttribute('class');
    if (!theClass) {
        node.className = newClass; // setAttribute('class', newClass);
    }
    else if (!theClass.match(newClass)) {
        node.className = theClass + ' ' + newClass; // setAttribute('class', theClass + ' ' + newClass);
    }
}

// Retire une classe CSS à un élément particulier
function removeClass(node, oldClass) {
    if (!node) return;
    var exp = new RegExp(oldClass, "g");
    var classes = node.className.replace(exp, "").split(/ +/);
    node.className = classes.join(" ");
}


// Cet objet _log gère toutes les traces (warning, debug...)
var _log = {
    // console: function() { return document.getElementById('logconsole'); },
    enable:  function() { GlobalConsole.style.display = 'block'; },
    disable: function() { GlobalConsole.style.display = 'none';  },
    setLevel: function(level) {
        for (var i=1; i<6; i++) {
            if (i<=level) { getStyleClass('log'+i).style.display = 'block'; }
            else          { getStyleClass('log'+i).style.display = 'none';  }
        }
    },
    disp: function(level, text) {
        // alert(text);
        var line = document.createElement('div');
        line.className = 'log'+level; //setAttribute('class', 'log'+level);
        line.innerHTML = '<strong>'+LOG_NAMES[level]+':</strong> '+text;
        GlobalConsole.appendChild(line);
        if (GlobalConsole.childNodes.length > LOG_MAX_SIZE) {
            GlobalConsole.removeChild(GlobalConsole.firstChild);
        }
    },
    debug:    function(text) { this.disp(LOG_DEBUG,    text); },
    info:     function(text) { this.disp(LOG_INFO,     text); },
    warning:  function(text) { this.disp(LOG_WARNING,  text); },
    error:    function(text) { this.disp(LOG_ERROR,    text); },
    critical: function(text) { this.disp(LOG_CRITICAL, text); }
}

// Permet de gérer le niveau de verbosité dans la console de logs.
// Le niveau (level) est récupéré à partir de la valeur de la combobox correspondante dans la page.
// Si level = 0, la console n'est pas affichée.
function setLogLevel() {
    // alert("coin");
    var level = parseInt(document.getElementById('config-log').value);
    if (level == 0) {
        _log.disable();
    }
    else {
        _log.enable();
        _log.setLevel(level);
    }
    // _log.info("Log level set to "+LOG_NAMES[level]);
}

// Ajout d'une méthode à l'objet Date pour retourner un timestamp au format YYYYMMDDhhmmss
function pad0(i) { return (i<10) ? '0'+i : ''+i ; }
Date.prototype.timestamp = function () {
    return this.getFullYear() + pad0(this.getMonth()+1) + pad0(this.getDate())
         + pad0(this.getHours()) + pad0(this.getMinutes()) + pad0(this.getSeconds());
}

// Une chtite fonction map pour les tableaux, fait suer de pas en avoir par défaut
function map(fun) {
    var len = this.length;
    var res = new Array(len);
    for (var i=0; i<len; i++) {
        res[i] = fun(this[i]);
    }
    return res;
}
Array.prototype.map = map;

// Pareil avec each
Array.prototype.each = function (fun) {
    for (var i=0, l=this.length; i<l; i++) {
        fun(this[i]);
    }
}

// Idem avec contains
Array.prototype.contains = function (value) {
    for (var i=this.length; i--;) {
        if (this[i] == value) {
            return true;
        }
    }
    return false;
}

// Et pis remove passque quand même
Array.prototype.remove = function (item) {
    for (var i=this.length; i--;) {
        if (this[i] === item) this.splice(i, 1);
    }
}

// Ca c'est un hack foireux pour rendre compatible les arrays avec les résultats de queries xpath
Array.prototype.getLength = function () { return this.length; };

// Gestion des cookies
function getCookie(name) {
    // _log.debug("doc.cookie= "+document.cookie);
    var nameEQ = name + "=";
    var ca = document.cookie.split(';');
    for (var i=ca.length;i--;) {
        var c = ca[i].strip();
        // while (c.charAt(0)==' ') c = c.substring(1, c.length);
        if (c.indexOf(nameEQ) == 0) {
            return unescape(c.substr(nameEQ.length));
        }
    }
    return null;
}

function setCookie(name, value, days) {
    if (days) {
        var date = new Date();
        date.setTime(date.getTime()+(days*24*60*60*1000));
        var expires = "; expires="+date.toGMTString();
    }
    else { var expires = "" };
    document.cookie = name+"="+escape(value)+expires+"; path=/";
    // _log.debug("cookie set: "+name+"="+escape(value)+expires+"; path=/");
}

function IE_selectNodes(classes, notclasses) {
    var all = GlobalPinni.getElementsByTagName("div") || [];
    if (!notclasses) notclasses = [];
    // _log.error('select IE: '+classes+', '+notclasses);
    var res = new Array();
    for (var i=all.length; i--;) {
        var curdiv = all[i];
        // var curclasses = curdiv.className.split(/\s+/);
        var bool = true;
        for (var j=classes.length; j--;) {
            if (curdiv.className.indexOf(classes[j]) == -1) {
               bool = false;
               break;
            }
        }
        if (bool) {
            for (var j=notclasses.length; j--;) {
                if (curdiv.className.indexOf(notclasses[j]) != -1) {
                   bool = false;
                   break;
                }
            }
        }
        if (bool) res.push(curdiv);
    }
    // _log.error('select IE result='+res);
    return res;
}

function evalexp(query) {
    // return document.evaluate(query, document.getElementById("pinnipede"), null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
    return evalXPath(document, query);
}

// retourne la position left et top d'un élément
function findPos(obj) {
    var curleft = curtop = 0;
    if (obj.offsetParent) {
        curleft = obj.offsetLeft;
        curtop = obj.offsetTop;
        while (obj = obj.offsetParent) {
            curleft += obj.offsetLeft;
            curtop += obj.offsetTop;
        }
    }
    return [curleft, curtop];
}


// Gestion des widgets des panneaux de configuration
function CheckBox(c_id, txt, val) {
    tr_el = document.createElement('tr');
    tr_el.className = 'subpanel'; // setAttribute('class','subpanel');

    td_el = document.createElement('td');
    td_el.className = 'subpanel'; // setAttribute('class','subpanel');
    td_el.innerHTML = txt;
    tr_el.appendChild(td_el);

    td_el = document.createElement('td');
    td_el.className = 'subpanel'; // setAttribute('class','subpanel');

    but = document.createElement('input');
    but.setAttribute('id',c_id);
    but.setAttribute('type','checkbox');
    if (val) {
        but.setAttribute('checked','yes');
    }

    td_el.appendChild(but);
    tr_el.appendChild(td_el);
    return tr_el;
}

function TextBox(c_id, txt, val, size) {
    tr_el = document.createElement('tr');
    tr_el.className = 'subpanel'; // setAttribute('class','subpanel');

    td_el = document.createElement('td');
    td_el.className = 'subpanel'; // setAttribute('class','subpanel');
    td_el.innerHTML = txt;
    tr_el.appendChild(td_el);

    td_el = document.createElement('td');
    td_el.className = 'subpanel'; // setAttribute('class','subpanel');

    but = document.createElement('input');
    but.setAttribute('id',c_id);
    but.setAttribute('type','textbox');
    but.setAttribute('size',size);
    but.setAttribute('value',val);

    td_el.appendChild(but);
    tr_el.appendChild(td_el);
    return tr_el;
}

function InfoBox(txt, val) {
    tr_el = document.createElement('tr');
    tr_el.className = 'subpanel'; // setAttribute('class','subpanel');

    td_el = document.createElement('td');
    td_el.className = 'subpanel'; // setAttribute('class','subpanel');
    td_el.innerHTML = txt;
    tr_el.appendChild(td_el);

    td_el = document.createElement('td');
    td_el.className = 'subpanel config-info';
    td_el.innerHTML = val;
    tr_el.appendChild(td_el);

    return tr_el;
}

function SelectBox(c_id, txt, tab, val, size) {
    tr_el = document.createElement('tr');
    tr_el.className = 'subpanel';

    td_el = document.createElement('td');
    td_el.className = 'subpanel';
    td_el.innerHTML = txt;
    tr_el.appendChild(td_el);

    td_el = document.createElement('td');
    td_el.className = 'subpanel';

    but = document.createElement('select');
    but.setAttribute('id',c_id);

    for(i=0; i<tab.length; i++) {
        opt = document.createElement('option');
        opt.innerHTML = tab[i];
        if (val == tab[i]) {
            opt.setAttribute('selected','yes');
        }
        but.appendChild(opt);
    }

    td_el.appendChild(but);
    tr_el.appendChild(td_el);
    return tr_el;
}

function to_url(chaine) {
    return str_replace('&', '%26', chaine);
}
function str_replace(p, r, s) {
    return s.replace(RegExp(encodeRE(p), 'g'), encodeRE(r));
}
function encodeRE(s) { // Theodor Zoulias, http://simon.incutio.com/archive/2006/01/20/escape#comment14
    return s.replace(/([.*+?^${}()|[\]\/\\])/g, '\\$1');
}

// ]/

String.prototype.strip = function () {
    return this.replace(/^\s+|\s+$/g, '');
}

// Evenements cross-browser
function addEvent(object, event, callback, flag) {
    if (is_ie) {
        object.attachEvent("on"+event, callback);
    }
    else {
        object.addEventListener(event, callback, flag);
    }
}

// Divers ajouts pour compatibilité IE/reste du monde
var srz = null;
var no_xpath = false;
if (is_ie) {
    // Array.prototype.snapshotItem = function (i) { return this[i]; };
    window.XMLHttpRequest = function () {
        return new ActiveXObject('Microsoft.XMLHTTP');
    }
}
else {
    try {
        XPathResult.prototype.__defineGetter__('length', function () { return this.snapshotLength; });
    }
    catch (err) {
        no_xpath = true;
    }
    srz = new XMLSerializer();
}

// Retourne un objet xml dom, fonction cross-browser
function loadXML(text) {
    if (is_ie) {
        var xmlDoc = new ActiveXObject("Microsoft.XMLDOM");
        xmlDoc.async = "false";
        xmlDoc.setProperty("SelectionLanguage", "XPath");
        // xmlDoc.evaluate = function (query) { return this.selectNodes(query); };
        xmlDoc.loadXML(text);
    }
    else {
        var parser = new DOMParser();
        var xmlDoc = parser.parseFromString(text, "text/xml");
    }
    return xmlDoc;
}

function evalXPath(node, query) {
    // _log.info("XPath query="+query);
    if (is_ie) {
        return node.selectNodes(query);
    }
    else {
        var target = (node == document) ? GlobalPinni : node;
        try {
            return node.evaluate(query, target, null, XPathResult.ORDERED_NODE_SNAPSHOT_TYPE, null);
        }
        catch (err) {
            // _log.error("Erreur XPath :<br />"+err);
            return [];
        }
    }
}

function getItem(nodes, i) {
    if (is_ie || no_xpath) {
        return nodes[i];
    }
    else {
        return nodes.snapshotItem(i);
    }
}

function getLength(nodes) {
    if (is_ie || no_xpath) {
        return nodes.length;
    }
    else {
        return nodes.snapshotLength;
    }
}

function serializeNodes(node, tag) {
    if (is_ie || no_xpath) {
        var content = node.xml || node.innerHTML;
        return content.replace(new RegExp('<'+tag+'>(.*)<\/'+tag+'>'), "$1");
    }
    else {
        node.childNodes.map = map;
        return node.childNodes.map(function(x){return srz.serializeToString(x);}).join("");
    }
}

function getNodeText(node) {
    if (is_ie) {
        if (node.firstChild) {
            return node.firstChild.nodeValue.strip();
        }
        else {
            return '';
        }
    }
    else {
        return node.textContent.strip();
    }
    // AFR: webkit utilise probablement innerHTML
}

